Unit Zoom;

{File:		Zoom
 Sort of:	Panel
 Version:	1.0
 Original:	Unknown
 Updated by:	Lon Voogd}

{use the procedures openIt or Closeit for the named actions

Function Issieopen : Boolean, true if open}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, Menus;

type
  TZoomStyles   = (zsCenter,zsWipeRight,zsWipeLeft,zsWipeDown,zsWipeUp);
  TZoomStates   = (stClosed,stOpen);
  TZoomPanel    = class(TCustomPanel)
  private
    FTimer      : TTimer;
    FZoomSpeed  : Integer;
    FZoomSteps  : Integer;
    FZoomStyle  : TZoomStyles;
    FZoomState  : TZoomStates;
    OpenRect    : TRect;

    ClosedRect  : TRect;
    {Left, top, width & height when open}
    OL,OT,OW,OH : Integer;
    {Left, top, width & height when closed}
    CL,CT,CW,CH : Integer;
    CurrStep    : Integer;
    ZoomDir     : Integer;
    procedure SetZoomSpeed(Value: Integer);
    procedure SetZoomSteps(Value: Integer);
    procedure SetZoomStyle(Value: TZoomStyles);
    procedure SetZoomState(Value: TZoomStates);
  protected
    procedure TimerTick(Sender: TObject);
    procedure ZoomThePanel;
    procedure RestartTimer;

    procedure CreateParams(var Params: TCreateParams); override;
    procedure SetOpenClosedRects(L,T,W,H: Integer);
    procedure OpenOrClose;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure OpenIt;
    procedure CloseIt;
    Function Issieopen : boolean;
  published
    property Align;
    property Alignment;
    property BevelInner;
    property BevelOuter;
    property BorderStyle;
    property BorderWidth;
    property Caption;
    property Color;
    property Ctl3D;

    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property Locked;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property ZoomSpeed : Integer read FZoomSpeed write SetZoomSpeed default 1000;
    property ZoomSteps : Integer read FZoomSteps write SetZoomSteps default 5;

    property ZoomStyle : TZoomStyles read FZoomStyle write SetZoomStyle default
zsCenter;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Additional', [TZoomPanel]);

end;

constructor TZoomPanel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  {ZoomSpeed controls the Speed at which the panel steps through its
  zooming. The actual millisecond delay used is 1000 divided by ZoomSpeed.
  Therefore setting ZoomSpeed to a higher number will result in less
  of a delay between steps and hence a faster zoom.}
  FZoomSpeed := 1000;
  {This defines the total number of steps that the panel will take to get
  to its final size and location}

  FZoomSteps := 5;
  CurrStep := -1;
  {ZoomStyle defines the effect used when zooming.}
  FZoomStyle := zsCenter;
  {Set the panel as Open}
  FZoomState := stOpen;
  {First, we need to create a timer that will control the pacing of the
  panel's Zoom effect.}
  FTimer := TTimer.Create(Self);
  {Set the procedure that will handle the timer. Every time the timer expires
  it will transfer control to this procedure. The procedure moves the panel a
  single step and then the timer starts over.}

  FTimer.OnTimer := TimerTick;
  {Set the timer as initially off. CreateParams turns it on}
  FTimer.Enabled := False;
  {Set the rate at which the panel opens or closes}
  FTimer.Interval := 1000 div ZoomSpeed;
end;

destructor TZoomPanel.Destroy;
begin
  {When the panel is destroyed, kill the timer}
  FTimer.Free;
  inherited Destroy;
end;

procedure TZoomPanel.TimerTick(Sender: TObject);
begin
  {Go and do a single zoom step}

  ZoomThePanel;
end;

procedure TZoomPanel.SetZoomSpeed(Value: Integer);
begin
  {ZoomSpeed must be at least 1 (slowest) and no greater than 1000 (top speed)}
  if FZoomSpeed <> Value then
    begin
      if Value > 1000 then FZoomSpeed := 1000
      else
        if Value < 1 then FZoomSpeed := 1
        else FZoomSpeed := Value;
      {Set the rate at which the panel opens or closes}
      If FTimer <> Nil then FTimer.Interval := 1000 div ZoomSpeed;

    end;
end;

procedure TZoomPanel.SetZoomSteps(Value: Integer);
begin
  {ZoomSteps must be at least 1 and no greater than 100}
  if FZoomSteps <> Value then
    if Value < 1 then FZoomSteps := 1
    else
      if Value > 100 then FZoomSteps := 100
      else FZoomSteps := Value;
end;

procedure TZoomPanel.SetZoomStyle(Value: TZoomStyles);
begin
  if FZoomStyle <> Value then
    begin
      FZoomStyle := Value;
      {Go set the sizes of the open and closed states of the panel}

      SetOpenClosedRects(Left,Top,Width,Height);
      {Do a Zoom in design mode so we can see the effect}
      If (csDesigning in ComponentState) and (CurrStep=-1) then OpenOrClose;
    end;
end;

procedure TZoomPanel.SetZoomState(Value: TZoomStates);
begin
  if FZoomState <> Value then FZoomState := Value;
end;

procedure TZoomPanel.ZoomThePanel;
var
  OW,OH,CW,HW : Integer;
begin
  If FZoomState = stOpen then Inc(CurrStep) else Dec(CurrStep);
  OW := OpenRect.Right-OpenRect.Left; {open width}

  OH := OpenRect.Bottom-OpenRect.Top; {open height}
  CW := Trunc(OW/ZoomSteps*CurrStep); {current width}
  CH := Trunc(OH/ZoomSteps*CurrStep); {current height}
  Case ZoomStyle of
    zsCenter    : with ClosedRect do
                    SetBounds(Left-CW div 2,Top-CH div 2,CW,CH);
    zsWipeRight : with ClosedRect do
                    SetBounds(Left,Top,CW,OH);
    zsWipeLeft  : with ClosedRect do
                    SetBounds(Right-CW,Top,CW,OH);
    zsWipeUp    : with ClosedRect do

                    SetBounds(Left,Top-CH,OW,CH);
    zsWipeDown  : with ClosedRect do
                    SetBounds(Left,Top,OW,CH);
  end;
  {Disable the timer if we have done our last zoom step}
  If (CurrStep >= ZoomSteps) or (CurrStep <= 0) Then
    begin
      FTimer.Enabled := False;
      CurrStep := -1;
    end;
end;

procedure TZoomPanel.SetOpenClosedRects(L,T,W,H: Integer);
begin
  {Save the size that the panel should be when open. I had to do this in
  the CreateParams method because the size of the control is not set until

  after the Create is complete - so that users could override the size.}
  OpenRect := Rect(L,T,L+W,T+H);
  {Save the size that the panel should be when closed.}
  Case ZoomStyle of
      {This is the default zooming behavior. It picks a spot in the exact
      center of the control as the center of the zoom effect. The panel
      will open from and close into this point.}
    zsCenter    : ClosedRect := Rect(L+W div 2,T+H div 2,L+W div 2,T+H div 2);
      {The panel will open like a "screen door" to the right}

    zsWipeRight : ClosedRect := Rect(L,T,L,T+H);
      {The panel will open like a "screen door" to the left}
    zsWipeLeft  : ClosedRect := Rect(L+W,T,L+W,T+H);
      {The panel will open like a "window shade" going up}
    zsWipeUp    : ClosedRect := Rect(L,T+H,L+W,T+H);
      {The panel will open like a "window shade" going down}
    zsWipeDown  : ClosedRect := Rect(L,T,L+W,T);
  end;
end;

procedure TZoomPanel.CreateParams(var Params: TCreateParams);

begin
  inherited CreateParams(Params);
  {Go set the sizes of the open and closed states of the panel}
  with Params do SetOpenClosedRects(Left,Top,Width,Height);
  {Activate the timer to start the zoom open}
  OpenOrClose;
end;

Procedure TZoomPanel.OpenOrClose;
begin
  if (FZoomState = stOpen) then OpenIt else CloseIt;
end;

procedure TZoomPanel.OpenIt;
begin
  FZoomState := stOpen;
  RestartTimer;
end;

procedure TZoomPanel.CloseIt;
begin
  FZoomState := stClosed;

  RestartTimer;
end;

Function Tzoompanel.issieopen : boolean;
begin
  If Fzoomstate = stOpen then
  result := true
  else result := false;
end;

procedure TZoomPanel.RestartTimer;
begin
  If FZoomState = stOpen then CurrStep := 0 else CurrStep := ZoomSteps;
  {Turn the timer on}
  FTimer.Enabled := True;
end;

end.
